<#
.SYNOPSIS
    This script checks for potential CFA issues in the logs from the resultOutputDir.
    It looks for CFA detections and taint events in the logs and constructs a list of CFADetection objects.
    It then adds some diagnotics messages and guidance for each process.
    It also constructs a list of files to scan again using MpCmdRunPath -TrustCheck -File $filepath.
    return is a list of CFADetection objects and a list of files to scan again.
.NOTES
    Author: Moe Ghasemi
    Copyright Microsoft Corporation 2024
#>

param (
    [string]$resultOutputDir,
    [string]$MpCmdRunPath,
    [switch]$verbose
)

function Test-AuthenticodeSignature($pathToCheck) {
    <#
    .SYNOPSIS
        Checks if a file is signed by Microsoft
    .DESCRIPTION
        Checks if a file is signed by Microsoft
    .PARAMETER pathToCheck
        The path to the file to check
    .PARAMETER resultOutputDir
        The path to the directory where the result file will be saved
    .PARAMETER outputDir
        The path to the directory where the result file will be saved
    #>
    $issuerInfo = "$resultOutputDir\issuerInfo.txt"
    [int]$OSBuild = [system.environment]::OSVersion.Version.Build
    if ($OSBuild -le "7601") {
        $LegacyOS = "True"
    }
    if ($pathToCheck) {
        if (Test-Path -path $pathToCheck -ErrorAction SilentlyContinue) {
            $AuthenticodeSig = (Get-AuthenticodeSignature -FilePath $pathToCheck)
            $cert = $AuthenticodeSig.SignerCertificate
            $FileInfo = (get-command $pathToCheck).FileVersionInfo
            $issuer = $cert.Issuer
            #OS is older than 2016 and some built-in processes will not be signed
            if (($OSBuild -lt 14393) -and (!$AuthenticodeSig.SignerCertificate)) {
                if (($FileInfo.CompanyName -eq "Microsoft Corporation")) {
                    return
                }
                else {
                    Write-Error "Script execution terminated because a process or script that does not have any signature was detected" | Out-File $issuerInfo -append
                    $pathToCheck | Out-File $issuerInfo -append
                    $AuthenticodeSig | Format-List * | Out-File $issuerInfo -append
                    $cert | Format-List * | Out-File $issuerInfo -append
                    [Environment]::Exit(1)
                }
            }
            #check if valid
            if ($AuthenticodeSig.Status -ne "Valid") {
                Write-Error "Script execution terminated because a process or script that does not have a valid Signature was detected" | Out-File $issuerInfo -append
                $pathToCheck | Out-File $issuerInfo -append
                $AuthenticodeSig | Format-List * | Out-File $issuerInfo -append
                $cert | Format-List * | Out-File $issuerInfo -append
                [Environment]::Exit(1)
            }
            #check issuer
            if (($issuer -ne "CN=Microsoft Code Signing PCA 2011, O=Microsoft Corporation, L=Redmond, S=Washington, C=US") -and ($issuer -ne "CN=Microsoft Windows Production PCA 2011, O=Microsoft Corporation, L=Redmond, S=Washington, C=US") -and ($issuer -ne "CN=Microsoft Code Signing PCA, O=Microsoft Corporation, L=Redmond, S=Washington, C=US") -and ($issuer -ne "CN=Microsoft Code Signing PCA 2010, O=Microsoft Corporation, L=Redmond, S=Washington, C=US") -and ($issuer -ne "CN=Microsoft Development PCA 2014, O=Microsoft Corporation, L=Redmond, S=Washington, C=US")) {
                Write-Error "Script execution terminated because a process or script that is not Microsoft signed was detected" | Out-File $issuerInfo -append
                $pathToCheck | Out-File $issuerInfo -append
                $AuthenticodeSig | Format-List * | Out-File $issuerInfo -append
                $cert | Format-List * | Out-File $issuerInfo -append
                [Environment]::Exit(1)
            }
            if ($AuthenticodeSig.IsOSBinary -ne "True") {
                #If revocation is offline then test below will fail
                if (!$LegacyOS) {
                    $IsOnline = (Get-NetConnectionProfile).IPv4Connectivity -like "*Internet*"
                }
                $EKUArray = @('1.3.6.1.5.5.7.3.3', '1.3.6.1.4.1.311.76.47.1')
                if ($IsOnline) {
                    $IsWindowsSystemComponent = (Test-Certificate -Cert $cert -EKU "1.3.6.1.4.1.311.10.3.6" -ErrorAction SilentlyContinue -WarningAction SilentlyContinue -WarningVariable OsCertWarnVar -ErrorVariable OsCertErrVar)
                    $IsMicrosoftPublisher = (Test-Certificate -Cert $cert -EKU "1.3.6.1.4.1.311.76.8.1" -ErrorAction SilentlyContinue -WarningAction SilentlyContinue -WarningVariable MsPublisherWarnVar -ErrorVariable MsPublisherErrVar)
                    $TrustedEKU = (Test-Certificate -Cert $cert -EKU $EKUArray -ErrorAction SilentlyContinue -WarningAction SilentlyContinue -WarningVariable EKUWarnVar -ErrorVariable EKUErrVar)
                    if (($IsWindowsSystemComponent -eq $False) -and ($IsMicrosoftPublisher -eq $False) -and ($TrustedEKU -eq $False)) {
                        #Defender AV and some OS processes will have an old signature if older version is installed
                        #Ignore if cert is OK and only signature is old
                        if (($OsCertWarnVar -like "*CERT_TRUST_IS_NOT_TIME_VALID*") -or ($MsPublisherWarnVar -like "*CERT_TRUST_IS_NOT_TIME_VALID*") -or ($OsCertWarnVar -like "*CERT_TRUST_IS_OFFLINE_REVOCATION*") -or ($MsPublisherWarnVar -like "CERT_TRUST_IS_OFFLINE_REVOCATION")) {
                            return
                        }
                        Write-Error "Script execution terminated because the process or script certificate failed trust check" | Out-File $issuerInfo -append
                        $pathToCheck | Out-File $issuerInfo -append
                        $AuthenticodeSig | Format-List * | Out-File $issuerInfo -append
                        $cert | Format-List * | Out-File $issuerInfo -append
                        [Environment]::Exit(1)
                    }
                }
            }
        }
     else {
            Write-Error ("Path " + $pathToCheck + " was not found") | Out-File $issuerInfo -append
        }
    }
}

# This script checks for potential CFA issues in the logs from the resultOutputDir.
# It looks for CFA detections and taint events in the logs and constructs a list of CFADetection objects.
# It then adds some diagnotics messages and guidance for each process.
# It also constructs a list of files to scan again using MpCmdRunPath -TrustCheck -File $filepath.
# return is a list of CFADetection objects and a list of files to scan again.
function Find-CfaDetections($resultOutputDir) {
    # Define new types for CFA issues:
    # TaintEvent: TaintType a number, taint reason a string:
    # CFAEvent: Action a string, Reason a number:
    # CFADetection: ProcessName a string, list of CFAEvents, list of TaintEvents:
    $csharpString = @"
    using System.Collections.Generic;

    public class TaintEvent {
        public int TaintType;
        public string TaintReason;
    }

    public class CFAEvent {
        public string Action;
        public int Reason;
    }

    public class DiagMessage {
        public string description;
        public string guidance;
    }

    public class CFADetection {
        public string ProcessName;
        public List<CFAEvent> CFAEvents = new List<CFAEvent>();
        public List<TaintEvent> TaintEvents = new List<TaintEvent>();
        public List<DiagMessage> DiagMessages = new List<DiagMessage>();
    }
"@

    # Check if the type already exists; if not, add it
    if (-not ([System.Management.Automation.PSTypeName]'CFAEvent').Type) {
        Add-Type -TypeDefinition $csharpString
    }

    $supportLogPath = Join-Path $resultOutputDir "DefenderAV"
    $supportLogPath = Join-Path $supportLogPath "MPSupportFiles.cab"
    if (test-path $supportLogPath -ErrorAction SilentlyContinue) {
        Write-Host "CFA-Check: Checking potential CFA issues using logs from $resultOutputDir"
    } else {
        Write-Host "CFA-Check: No logs found. exiting."
        return
    }
    Write-Host "CFA-Check: Checking potential CFA issues using logs from $resultOutputDir"
    Start-Process -wait -WindowStyle minimized "expand" -ArgumentList "-I $supportLogPath -F:MPLog* $resultOutputDir\DefenderAV"
    $mpLogFiles = Get-ChildItem -Path $resultOutputDir\DefenderAV -Filter "MPLog*.log"

    # A list of CFADetection objects per unique process name:
    # holds list of processes with CFA detections(as list of CFADetetion objects):
    $cfaDetections = @()

    $mpLogFiles | ForEach-Object {
        $mpLogFile = $_.FullName
        Write-Host "CFA-Check: Checking $mpLogFile for CFA detections:"
        $mpLogContent = Get-Content -Path $mpLogFile
        # Need to looks for this regex and extract process name and reason:
        # ASR Notification:Process: ([\w:\\\s.\%-]+),\sAction:\s(BLOCK|AUDIT),\sTarget:\s[\w\%\\.\s:-]+,\sReason:\s(\d)+,\sPersisted:\s(y|n),\sRule:0x5737d832
        $mpLogContent | Select-String -Pattern "ASR Notification:Process: ([\w:\\\s.\%-]+),\sAction:\s(BLOCK|AUDIT),\sTarget:\s[\w\%\\.\s:-]+,\sReason:\s(\d)+,\sPersisted:\s(y|n),\sRule:0x5737d832" -AllMatches | ForEach-Object {
            $processName = $_.Matches.Groups[1].Value
            $action = $_.Matches.Groups[2].Value
            $reason = $_.Matches.Groups[3].Value
            if ($verbose) {
                Write-Host "CFA-Check: Found CFA detection: Process: $processName, Action: $action, Reason: $reason"
            }
            # add this detection to the list of detections:
            $cfaDetection = $cfaDetections | Where-Object { $_.ProcessName -eq $processName }
            if ($cfaDetection -eq $null) {
                $cfaDetection = New-Object CFADetection
                $cfaDetection.ProcessName = $processName
                $cfaDetections += $cfaDetection
            }
            $cfaEvent = New-Object CFAEvent
            $cfaEvent.Action = $action
            $cfaEvent.Reason = $reason
            $cfaDetection.CFAEvents.Add($cfaEvent)
        }
    }

    # now iterate over files again to look for taint events:
    # using this regex: Engine:Process\s([\w:\\\s.\%-]+)\s\(PPID:\d+:\d+\)\sis\stainted:\sTaintType:0x\d+[.]\sTaintReason:([\w:\\\s.\%-]+)$
    $mpLogFiles | ForEach-Object {
        $mpLogFile = $_.FullName
        Write-Host "CFA-Check: Checking $mpLogFile for taint events:"
        $mpLogContent = Get-Content -Path $mpLogFile
        $mpLogContent | Select-String -Pattern "Engine:Process\s([\w:\\\s.\%-]+)\s\(PPID:\d+:\d+\)\sis\stainted:\sTaintType:0x(\d)+[.]\sTaintReason:([\w:\\\s.\%-]+)$" -AllMatches | ForEach-Object {
            $processName = $_.Matches.Groups[1].Value
            $taintType = $_.Matches.Groups[2].Value
            $taintReason = $_.Matches.Groups[3].Value
            if ($verbose) {
                Write-Host "Found taint event: Process: $processName, TaintType:$taintType TaintReason: $taintReason"
            }
            # add this taint event to the list of detections only if the process is in the list of CFA detections:
            $cfaDetection = $cfaDetections | Where-Object { $_.ProcessName -eq $processName }
            if ($cfaDetection -ne $null) {
                $taintEvent = New-Object TaintEvent
                $taintEvent.TaintType = [int]$taintType
                $taintEvent.TaintReason = $taintReason
                $cfaDetection.TaintEvents.Add($taintEvent)
            }
        }
    }
    # some literal strings for constructing recommendations:
    $untrustedFileMessage = "Add an AV exclusion for this file if you trust this file."
    $untrustedProcessMessage = "Add an AV exclusion for this process if you trust it(Process Exclusion)."
    # A list of file to scan again:
    $cfaDiagsFile = Join-Path $resultOutputDir "DefenderAV\CfaDiags.txt"
    $filesToScan = @()
    "CFA-Check: List of CFA detections Founds:" | Out-File -FilePath $cfaDiagsFile
    $cfaDetections | ForEach-Object {
        $processName = $_.ProcessName
        $cfaDetection = $_
        "    Process: $processName" | Out-File -FilePath $cfaDiagsFile -Append
        Write-Host "        CFA Events:" | Out-File -FilePath $cfaDiagsFile -Append
        $_.CFAEvents | ForEach-Object {
            Write-Host "            Action: $($_.Action), Reason: ($($_.Reason))" | Out-File -FilePath $cfaDiagsFile -Append
            if ($_.Reason -eq 1) {
                # unfriendly process image, add info to diagmessage.
                $desc = "$processName is not trusted by CFA because process image is not known good."
                $diagMessage = $cfaDetection.DiagMessages | Where-Object { $_.description -eq $desc }
                if ($diagMessage -eq $null) {
                    $diagMessage = New-Object DiagMessage
                    $diagMessage.description = $desc
                    $diagMessage.guidance = "Add it to list of allowed apps or " + $untrustedFileMessage 
                    $cfaDetection.DiagMessages.Add($diagMessage)
                    "                DiagMessage: $($diagMessage.description)" | Out-File -FilePath $cfaDiagsFile -Append
                    "                Guidance: $($diagMessage.guidance)" | Out-File -FilePath $cfaDiagsFile -Append
                }
                # Add processname to filesToScan if it is not already there.
                if ($filesToScan -notcontains $processName) {
                    $filesToScan += $processName
                }
            }
            if ($_.Reason -eq 2) {
                # parent unfriendly:
                $desc = "$processName is not trusted by CFA because its parent is not known good."
                $diagMessage = $cfaDetection.DiagMessages | Where-Object { $_.description -eq $desc }
                if ($diagMessage -eq $null) {
                    $diagMessage = New-Object DiagMessage
                    $diagMessage.description = $desc
                    $diagMessage.guidance = "N/A"
                    $cfaDetection.DiagMessages.Add($diagMessage)
                    "                DiagMessage: $($diagMessage.description)" | Out-File -FilePath $cfaDiagsFile -Append
                    "                Guidance: $($diagMessage.guidance)" | Out-File -FilePath $cfaDiagsFile -Append
                }
            }
            if ($_.Reason -eq 4) {
                # parent sript:
                $desc = "$processName is not trusted by CFA because its parent is a script."
                $diagMessage = $cfaDetection.DiagMessages | Where-Object { $_.description -eq $desc }
                if ($diagMessage -eq $null) {
                    $diagMessage = New-Object DiagMessage
                    $diagMessage.description = $desc
                    $diagMessage.guidance = "N/A"
                    $cfaDetection.DiagMessages.Add($diagMessage)
                    "                DiagMessage: $($diagMessage.description)" | Out-File -FilePath $cfaDiagsFile -Append
                    "                Guidance: $($diagMessage.guidance)" | Out-File -FilePath $cfaDiagsFile -Append
                }
            }
        }
        "        Taint Events:" | Out-File -FilePath $cfaDiagsFile -Append
        $_.TaintEvents | ForEach-Object {
            "            TaintType: $($_.TaintType)" | Out-File -FilePath $cfaDiagsFile -Append
            "            TaintReason: $($_.TaintReason)" | Out-File -FilePath $cfaDiagsFile -Append
            # if tainttype is 1, add a reccommendation to exclude taintreason:
            if ($_.TaintType -eq 1) {
                # loaded untrusted file, add info to diagmessage.
                $desc = "$processName is not trusted by CFA because it load an untrusted file: $($_.TaintReason)."
                $diagMessage = $cfaDetection.DiagMessages | Where-Object { $_.description -eq $desc }
                if ($diagMessage -eq $null) {
                    $diagMessage = New-Object DiagMessage
                    $diagMessage.description = $desc
                    $diagMessage.guidance = "$untrustedFileMessage"
                    $cfaDetection.DiagMessages.Add($diagMessage)
                    "                DiagMessage: $($diagMessage.description)" | Out-File -FilePath $cfaDiagsFile -Append
                    "                Guidance: $($diagMessage.guidance)" | Out-File -FilePath $cfaDiagsFile -Append
                }
            }
            if ($_.TaintType -ge 2 -and $_.TaintType -le 8) {
                # if taint type is 2 to 8, add a reccommendation to exclude taintreason:
                $desc = "$processName is not trusted by CFA because it was injected into by an untrusted process: $($_.TaintReason)."
                $diagMessage = $cfaDetection.DiagMessages | Where-Object { $_.description -eq $desc }
                if ($diagMessage -eq $null) {
                    $diagMessage = New-Object DiagMessage
                    $diagMessage.description = $desc
                    $diagMessage.guidance = "$untrustedProcessMessage"
                    $cfaDetection.DiagMessages.Add($diagMessage)
                    "                DiagMessage: $($diagMessage.description)" | Out-File -FilePath $cfaDiagsFile -Append
                    "                Guidance: $($diagMessage.guidance)" | Out-File -FilePath $cfaDiagsFile -Append
                }
            }
            if ($_.TaintType -ge 1 -and $_.TaintType -le 8) {
                # Add taint reason to list of files for scanning.
                if ($filesToScan -notcontains $_.TaintReason) {
                    $filesToScan += $_.TaintReason
                }
            }
        }
        "###############################################" | Out-File -FilePath $cfaDiagsFile -Append
    }
    # return the list of CFA detections and the list of files to scan again:
    return $cfaDetections, $filesToScan
}

function Test-ScanCfaReasons($filesToScan, $MpCmdRunPath, $resultOutputDir) {
    # check for empty mpcmdrunpath:
    if (-not $MpCmdRunPath) {
        Write-Host "ScanCFAReasons: MpCmdRunPath is not set, cannot scan files again."
        return
    }
    # check for empty list of files to scan:
    if ($filesToScan.Count -eq 0) {
        Write-Host "ScanCFAReasons: No files to scan again."
        return
    }
    Write-Host "ScanCFAReasons: Scanning files again using MpCmdRun"

    # create a temp file in temp folder to send stdin to the process:
    $stdinTmpFile = [System.IO.Path]::GetTempFileName()
    # open this file for writing only with shared read access:
    $mpCmdRunStdIn = New-Object System.IO.FileStream($stdinTmpFile, [System.IO.FileMode]::Open, [System.IO.FileAccess]::Write, [System.IO.FileShare]::ReadWrite)
    # create a temp file for std out:
    $stdoutTmpFile = [System.IO.Path]::GetTempFileName()
    # Start a new background process that start log collection using MpCmdRunPath -Trace -Grouping 0x2 -Level 0X1f. we need to come back to this process to stop tracing and copy the logs.
    # pass the stdout of the process to a variable.
    Test-AuthenticodeSignature $MpCmdRunCommand
    $logCollectionProcess = Start-Process -PassThru -FilePath $MpCmdRunPath -ArgumentList "-Trace -Grouping 0x2 -Level 0X1f" -NoNewWindow -RedirectStandardInput $stdinTmpFile -RedirectStandardOutput $stdoutTmpFile
    # sleep for 2 seconds to let the process start.
    Start-Sleep -Seconds 2
    # Scan files one by one using MpCmdRunPath -TrustCheck -File $filepath, write output of commands to a file.
    $cfaDiagsFile = Join-Path $resultOutputDir "DefenderAV\CfaDiags.txt"
    $filesToScan | ForEach-Object {
        $filePath = $_
        Write-Host "ScanCFAReasons: Scanning file: $filePath"
        # Write file path to CfaTrustChecks.txt:
        "#### Scanning $filePath" | Out-File -FilePath "$cfaDiagsFile" -Append
        # check if file exists:
        if (-not (Test-Path -Path $filePath)) {
            "File not found: $filePath" | Out-File -FilePath "$cfaDiagsFile" -Append
        } else {
            # run command and append output to a file.
            Test-AuthenticodeSignature $MpCmdRunCommand
            $trustCheckOutput = Invoke-Expression -Command "& '$MpCmdRunPath' -TrustCheck -File '$filePath'"
            $trustCheckOutput | Out-File -FilePath "$cfaDiagsFile" -Append
            Get-FileHash -Path $filePath | Out-File -FilePath "$cfaDiagsFile" -Append
        }
    }
    # sleep for 3 seconds to let all logs be collected:
    Start-Sleep -Seconds 3
    # stop the process that was capturing logs by sending an ENTER character to file.
    $mpCmdRunStdIn.Write([byte[]]@(0x00, 0x0d, 0x00, 0x0a), 0, 4)
    $mpCmdRunStdIn.Flush()
    $mpCmdRunStdIn.Close()
    # wait for process to exit.
    $logCollectionProcess.WaitForExit()
    # read the stdout of the process from the file.
    $logCollectionStdOut = Get-Content -Path $stdoutTmpFile
    # get path of mptrace log file from stdout using regex: Trace File location:\s([\w:\\\s.\%-]+)$
    $mpTraceLogFile = $logCollectionStdOut | Select-String -Pattern "Trace File location:\s([\w:\\\s.\%-]+)$" -AllMatches | ForEach-Object {
        $_.Matches.Groups[1].Value
    }

    # copy the mptrace log file to resultsoutputdir.
    Copy-Item -Path $mpTraceLogFile -Destination "$resultOutputDir\DefenderAV\MpTrace-CfaScanReasons.bin"
}

#main:
Export-ModuleMember -Function Find-CfaDetections, Test-ScanCfaReasons

# SIG # Begin signature block
# MIIn4AYJKoZIhvcNAQcCoIIn0TCCJ80CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBPBCzug8wk7ljz
# xJJYxj8ZIj8nEMCpQxdvrR4vUJy1RaCCDZcwggYVMIID/aADAgECAhMzAAADz7d0
# vUQxddCTAAAAAAPPMA0GCSqGSIb3DQEBCwUAMH4xCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25p
# bmcgUENBIDIwMTEwHhcNMjQwMjIyMTkwNzUzWhcNMjUwMjE5MTkwNzUzWjCBlDEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
# bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjE+MDwGA1UEAxM1TWlj
# cm9zb2Z0IFdpbmRvd3MgRGVmZW5kZXIgQWR2YW5jZWQgVGhyZWF0IFByb3RlY3Rp
# b24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDwAPmxMmR9a92NX/Lv
# 2PmaySDZqZUANxu28n+GqAuhMRrC8/v5HrKeebKt+6wqnp28fO1jvJ5OlBIrirN7
# KUiSUH86mvByCFnIwIkKcusZbgEF+v15S0jRExa5C9lC+kDVYtlSCnPEdx/tkKLN
# QQMmSroC3baj7y0WIvBfZ1l4x2qyDhFC/5plmMfD8TSfl3sN4twBiXAcpJ/SES50
# QbLahl/MTsVA2UEo7ygPJIrmci8wvD0Kt1pfkwbB3eiK4Vatr0VoJujRucjGT322
# m/9vH6p4Y5suUJMo9196/ze8hA8bqH+5NdU+DeRLrV/UzVat1Xs/Wm6kPK8GgTPL
# H+M9AgMBAAGjggFzMIIBbzAfBgNVHSUEGDAWBggrBgEFBQcDAwYKKwYBBAGCN0wv
# ATAdBgNVHQ4EFgQU10vjUl0bvgJvpCFtGT8ECb23+CAwRQYDVR0RBD4wPKQ6MDgx
# HjAcBgNVBAsTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEWMBQGA1UEBRMNNDUxODk0
# KzUwMjA2OTAfBgNVHSMEGDAWgBRIbmTlUAXTgqoXNzcitW2oynUClTBUBgNVHR8E
# TTBLMEmgR6BFhkNodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpb3BzL2NybC9N
# aWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3JsMGEGCCsGAQUFBwEBBFUwUzBR
# BggrBgEFBQcwAoZFaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9wcy9jZXJ0
# cy9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3J0MAwGA1UdEwEB/wQCMAAw
# DQYJKoZIhvcNAQELBQADggIBAHcmfqgz/boPmZhEOAtYI5wj3mRicbSQwDQcGZfK
# 8I1WdILE80dhNGqgtVvxO5v5/PyjBJY5HV9HxxCzdVVj60PYAwzoyfTIZF+0hPrh
# l6JkjfckCiy9qsONVkL6SDJH6a+Xf4penA2ge9WM3XLR5BUQH538wx4w+VswV1QX
# hKsAMGAvxBG8EzIFTMRXMoKy/RYZ1FSig96mdJRb9Zh4o37XcyAgSxlbiv50vu3X
# MGNXRZJY0/PdB6Du55mI0WfJDaKBJuDShBMLOjDRX8UEWjy3rqGTjVVLYhK4mWq6
# I/1bIZlgkKBFnUMFbVvtSWDP13FgyGIVDk5YB5s2XmSC6hTfXAHakSpLfFeZ1VK5
# /pRMdhxEenleDdsgksd9idt+U7YxgjZ6gfdGuWgj9ZNi8Ef4ILoA5fkUGIIP14a7
# j2SUsaYvJKw/5aiVHRza+Q9gbDpNcsR5WbhhcSAnkFmi7ZkmJKHVlkUqAUDbP8WB
# yz/bq6m/HCHELfFfsG4HMvOfLSTVhKfXPptRzN4N5m2AZX423KM2zhqzPjCWNWce
# /jqhUKtbyMuPPqqo+eaoYi+ZzaEqirxAmPF6Nz3VIncZppzm451L2Uu05NwN1T5m
# ZbiGW2WW1Iz50ofUKACCqThHMtSh4fOdq0OiKaqQz3S5BGdoXdPPOKgXxOtfaMpr
# C3SrMIIHejCCBWKgAwIBAgIKYQ6Q0gAAAAAAAzANBgkqhkiG9w0BAQsFADCBiDEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
# bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMpTWlj
# cm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5IDIwMTEwHhcNMTEwNzA4
# MjA1OTA5WhcNMjYwNzA4MjEwOTA5WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
# V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
# IENvcnBvcmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
# QSAyMDExMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAq/D6chAcLq3Y
# bqqCEE00uvK2WCGfQhsqa+laUKq4BjgaBEm6f8MMHt03a8YS2AvwOMKZBrDIOdUB
# FDFC04kNeWSHfpRgJGyvnkmc6Whe0t+bU7IKLMOv2akrrnoJr9eWWcpgGgXpZnbo
# MlImEi/nqwhQz7NEt13YxC4Ddato88tt8zpcoRb0RrrgOGSsbmQ1eKagYw8t00CT
# +OPeBw3VXHmlSSnnDb6gE3e+lD3v++MrWhAfTVYoonpy4BI6t0le2O3tQ5GD2Xuy
# e4Yb2T6xjF3oiU+EGvKhL1nkkDstrjNYxbc+/jLTswM9sbKvkjh+0p2ALPVOVpEh
# NSXDOW5kf1O6nA+tGSOEy/S6A4aN91/w0FK/jJSHvMAhdCVfGCi2zCcoOCWYOUo2
# z3yxkq4cI6epZuxhH2rhKEmdX4jiJV3TIUs+UsS1Vz8kA/DRelsv1SPjcF0PUUZ3
# s/gA4bysAoJf28AVs70b1FVL5zmhD+kjSbwYuER8ReTBw3J64HLnJN+/RpnF78Ic
# V9uDjexNSTCnq47f7Fufr/zdsGbiwZeBe+3W7UvnSSmnEyimp31ngOaKYnhfsi+E
# 11ecXL93KCjx7W3DKI8sj0A3T8HhhUSJxAlMxdSlQy90lfdu+HggWCwTXWCVmj5P
# M4TasIgX3p5O9JawvEagbJjS4NaIjAsCAwEAAaOCAe0wggHpMBAGCSsGAQQBgjcV
# AQQDAgEAMB0GA1UdDgQWBBRIbmTlUAXTgqoXNzcitW2oynUClTAZBgkrBgEEAYI3
# FAIEDB4KAFMAdQBiAEMAQTALBgNVHQ8EBAMCAYYwDwYDVR0TAQH/BAUwAwEB/zAf
# BgNVHSMEGDAWgBRyLToCMZBDuRQFTuHqp8cx0SOJNDBaBgNVHR8EUzBRME+gTaBL
# hklodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtpL2NybC9wcm9kdWN0cy9NaWNS
# b29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3JsMF4GCCsGAQUFBwEBBFIwUDBOBggr
# BgEFBQcwAoZCaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNS
# b29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3J0MIGfBgNVHSAEgZcwgZQwgZEGCSsG
# AQQBgjcuAzCBgzA/BggrBgEFBQcCARYzaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
# L3BraW9wcy9kb2NzL3ByaW1hcnljcHMuaHRtMEAGCCsGAQUFBwICMDQeMiAdAEwA
# ZQBnAGEAbABfAHAAbwBsAGkAYwB5AF8AcwB0AGEAdABlAG0AZQBuAHQALiAdMA0G
# CSqGSIb3DQEBCwUAA4ICAQBn8oalmOBUeRou09h0ZyKbC5YR4WOSmUKWfdJ5DJDB
# ZV8uLD74w3LRbYP+vj/oCso7v0epo/Np22O/IjWll11lhJB9i0ZQVdgMknzSGksc
# 8zxCi1LQsP1r4z4HLimb5j0bpdS1HXeUOeLpZMlEPXh6I/MTfaaQdION9MsmAkYq
# wooQu6SpBQyb7Wj6aC6VoCo/KmtYSWMfCWluWpiW5IP0wI/zRive/DvQvTXvbiWu
# 5a8n7dDd8w6vmSiXmE0OPQvyCInWH8MyGOLwxS3OW560STkKxgrCxq2u5bLZ2xWI
# UUVYODJxJxp/sfQn+N4sOiBpmLJZiWhub6e3dMNABQamASooPoI/E01mC8CzTfXh
# j38cbxV9Rad25UAqZaPDXVJihsMdYzaXht/a8/jyFqGaJ+HNpZfQ7l1jQeNbB5yH
# PgZ3BtEGsXUfFL5hYbXw3MYbBL7fQccOKO7eZS/sl/ahXJbYANahRr1Z85elCUtI
# EJmAH9AAKcWxm6U/RXceNcbSoqKfenoi+kiVH6v7RyOA9Z74v2u3S5fi63V4Guzq
# N5l5GEv/1rMjaHXmr/r8i+sLgOppO6/8MO0ETI7f33VtY5E90Z1WTk+/gFcioXgR
# MiF670EKsT/7qMykXcGhiJtXcVZOSEXAQsmbdlsKgEhr/Xmfwb1tbWrJUnMTDXpQ
# zTGCGZ8wghmbAgEBMIGVMH4xCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5n
# dG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9y
# YXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENBIDIwMTEC
# EzMAAAPPt3S9RDF10JMAAAAAA88wDQYJYIZIAWUDBAIBBQCgga4wGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# LwYJKoZIhvcNAQkEMSIEIBuJ7KPJtiWSSOs2if9xFzCp9L4yqcGXJnxYSKoJsAuY
# MEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8AcwBvAGYAdKEagBhodHRw
# Oi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEBBQAEggEAXvK3CLgpJBQJ
# eHow39/eWf1FEzm0+yBYom6DZ/x/nwOp2FvJT1lWxsQrRbtlHDtfURQSKzw807tZ
# Uon3EV/IVtHG1MyNZ/WC9gFh5UVTZq1tIOx1xUCvKHekWQ73bwKKc9Qe/t/MdAQP
# Xt8ZLJ2umLPq2IKDULIUjFG4FrtAQ3XtWSJpdo7SZQgBZyHzDs/UxbgwJWLBkRHo
# ldnR/5q+d2AtDJsJ47BuHkJoF8JtmotS5RVRUj3zZiG5kYMINBIOIBHG3s82UVQn
# ySt/y16XJaRWXoimmjVc4s86hSMUNvG8Lzym4LzvKyumv+hH9UT27pE59hxn64Mj
# qlCU7fOUwqGCFykwghclBgorBgEEAYI3AwMBMYIXFTCCFxEGCSqGSIb3DQEHAqCC
# FwIwghb+AgEDMQ8wDQYJYIZIAWUDBAIBBQAwggFZBgsqhkiG9w0BCRABBKCCAUgE
# ggFEMIIBQAIBAQYKKwYBBAGEWQoDATAxMA0GCWCGSAFlAwQCAQUABCB0vK3t9yby
# iyK5DpU9sPBdYKhhuxWi9JoNZP/hO8t+DgIGZjOs1lftGBMyMDI0MDUxMjExMDg1
# NC41MDNaMASAAgH0oIHYpIHVMIHSMQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2Fz
# aGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENv
# cnBvcmF0aW9uMS0wKwYDVQQLEyRNaWNyb3NvZnQgSXJlbGFuZCBPcGVyYXRpb25z
# IExpbWl0ZWQxJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOkQwODItNEJGRC1FRUJB
# MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFtcCBTZXJ2aWNloIIReDCCBycw
# ggUPoAMCAQICEzMAAAHcweCMwl9YXo4AAQAAAdwwDQYJKoZIhvcNAQELBQAwfDEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
# bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWlj
# cm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwHhcNMjMxMDEyMTkwNzA2WhcNMjUw
# MTEwMTkwNzA2WjCB0jELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
# EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
# bjEtMCsGA1UECxMkTWljcm9zb2Z0IElyZWxhbmQgT3BlcmF0aW9ucyBMaW1pdGVk
# MSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjpEMDgyLTRCRkQtRUVCQTElMCMGA1UE
# AxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZTCCAiIwDQYJKoZIhvcNAQEB
# BQADggIPADCCAgoCggIBAIvIsyA1sjg9kSKJzelrUWF5ShqYWL83amn3SE5JyIVP
# UC7F6qTcLphhHZ9idf21f0RaGrU8EHydF8NxPMR2KVNiAtCGPJa8kV1CGvn3beGB
# 2m2ltmqJanG71mAywrkKATYniwKLPQLJ00EkXw5TSwfmJXbdgQLFlHyfA5Kg+pUs
# JXzqumkIvEr0DXPvptAGqkdFLKwo4BTlEgnvzeTfXukzX8vQtTALfVJuTUgRU7zo
# P/RFWt3WagahZ6UloI0FC8XlBQDVDX5JeMEsx7jgJDdEnK44Y8gHuEWRDq+SG9Xo
# 0GIOjiuTWD5uv3vlEmIAyR/7rSFvcLnwAqMdqcy/iqQPMlDOcd0AbniP8ia1BQEU
# nfZT3UxyK9rLB/SRiKPyHDlg8oWwXyiv3+bGB6dmdM61ur6nUtfDf51lPcKhK4Vo
# 83pOE1/niWlVnEHQV9NJ5/DbUSqW2RqTUa2O2KuvsyRGMEgjGJA12/SqrRqlvE2f
# iN5ZmZVtqSPWaIasx7a0GB+fdTw+geRn6Mo2S6+/bZEwS/0IJ5gcKGinNbfyQ1xr
# vWXPtXzKOfjkh75iRuXourGVPRqkmz5UYz+R5ybMJWj+mfcGqz2hXV8iZnCZDBrr
# nZivnErCMh5Flfg8496pT0phjUTH2GChHIvE4SDSk2hwWP/uHB9gEs8p/9Pe/mt9
# AgMBAAGjggFJMIIBRTAdBgNVHQ4EFgQU6HPSBd0OfEX3uNWsdkSraUGe3dswHwYD
# VR0jBBgwFoAUn6cVXQBeYl2D9OXSZacbUzUZ6XIwXwYDVR0fBFgwVjBUoFKgUIZO
# aHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9wcy9jcmwvTWljcm9zb2Z0JTIw
# VGltZS1TdGFtcCUyMFBDQSUyMDIwMTAoMSkuY3JsMGwGCCsGAQUFBwEBBGAwXjBc
# BggrBgEFBQcwAoZQaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9wcy9jZXJ0
# cy9NaWNyb3NvZnQlMjBUaW1lLVN0YW1wJTIwUENBJTIwMjAxMCgxKS5jcnQwDAYD
# VR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAOBgNVHQ8BAf8EBAMC
# B4AwDQYJKoZIhvcNAQELBQADggIBANnrb8Ewr8eX/H1sKt3rnwTDx4AqgHbkMNQo
# +kUGwCINXS3y1GUcdqsK/R1g6Tf7tNx1q0NpKk1JTupUJfHdExKtkuhHA+82lT7y
# ISp/Y74dqJ03RCT4Q+8ooQXTMzxiewfErVLt8WefebncST0i6ypKv87pCYkxM24b
# bqbM/V+M5VBppCUs7R+cETiz/zEA1AbZL/viXtHmryA0CGd+Pt9c+adsYfm7qe5U
# MnS0f/YJmEEMkEqGXCzyLK+dh+UsFi0d4lkdcE+Zq5JNjIHesX1wztGVAtvX0DYD
# ZdN2WZ1kk+hOMblUV/L8n1YWzhP/5XQnYl03AfXErn+1Eatylifzd3ChJ1xuGG76
# YbWgiRXnDvCiwDqvUJevVRY1qy4y4vlVKaShtbdfgPyGeeJ/YcSBONOc0DNTWbjM
# bL50qeIEC0lHSpL2rRYNVu3hsHzG8n5u5CQajPwx9PzpsZIeFTNHyVF6kujI4Vo9
# NvO/zF8Ot44IMj4M7UX9Za4QwGf5B71x57OjaX53gxT4vzoHvEBXF9qCmHRgXBLb
# RomJfDn60alzv7dpCVQIuQ062nyIZKnsXxzuKFb0TjXWw6OFpG1bsjXpOo5DMHky
# sribxHor4Yz5dZjVyHANyKo0bSrAlVeihcaG5F74SZT8FtyHAW6IgLc5w/3D+R1o
# bDhKZ21WMIIHcTCCBVmgAwIBAgITMwAAABXF52ueAptJmQAAAAAAFTANBgkqhkiG
# 9w0BAQsFADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAO
# BgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEy
# MDAGA1UEAxMpTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5IDIw
# MTAwHhcNMjEwOTMwMTgyMjI1WhcNMzAwOTMwMTgzMjI1WjB8MQswCQYDVQQGEwJV
# UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
# ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGlt
# ZS1TdGFtcCBQQ0EgMjAxMDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIB
# AOThpkzntHIhC3miy9ckeb0O1YLT/e6cBwfSqWxOdcjKNVf2AX9sSuDivbk+F2Az
# /1xPx2b3lVNxWuJ+Slr+uDZnhUYjDLWNE893MsAQGOhgfWpSg0S3po5GawcU88V2
# 9YZQ3MFEyHFcUTE3oAo4bo3t1w/YJlN8OWECesSq/XJprx2rrPY2vjUmZNqYO7oa
# ezOtgFt+jBAcnVL+tuhiJdxqD89d9P6OU8/W7IVWTe/dvI2k45GPsjksUZzpcGkN
# yjYtcI4xyDUoveO0hyTD4MmPfrVUj9z6BVWYbWg7mka97aSueik3rMvrg0XnRm7K
# MtXAhjBcTyziYrLNueKNiOSWrAFKu75xqRdbZ2De+JKRHh09/SDPc31BmkZ1zcRf
# NN0Sidb9pSB9fvzZnkXftnIv231fgLrbqn427DZM9ituqBJR6L8FA6PRc6ZNN3SU
# HDSCD/AQ8rdHGO2n6Jl8P0zbr17C89XYcz1DTsEzOUyOArxCaC4Q6oRRRuLRvWoY
# WmEBc8pnol7XKHYC4jMYctenIPDC+hIK12NvDMk2ZItboKaDIV1fMHSRlJTYuVD5
# C4lh8zYGNRiER9vcG9H9stQcxWv2XFJRXRLbJbqvUAV6bMURHXLvjflSxIUXk8A8
# FdsaN8cIFRg/eKtFtvUeh17aj54WcmnGrnu3tz5q4i6tAgMBAAGjggHdMIIB2TAS
# BgkrBgEEAYI3FQEEBQIDAQABMCMGCSsGAQQBgjcVAgQWBBQqp1L+ZMSavoKRPEY1
# Kc8Q/y8E7jAdBgNVHQ4EFgQUn6cVXQBeYl2D9OXSZacbUzUZ6XIwXAYDVR0gBFUw
# UzBRBgwrBgEEAYI3TIN9AQEwQTA/BggrBgEFBQcCARYzaHR0cDovL3d3dy5taWNy
# b3NvZnQuY29tL3BraW9wcy9Eb2NzL1JlcG9zaXRvcnkuaHRtMBMGA1UdJQQMMAoG
# CCsGAQUFBwMIMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIB
# hjAPBgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP6KJcYmjRPZSQW9fO
# mhjEMFYGA1UdHwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9w
# a2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIzLmNybDBaBggr
# BgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6Ly93d3cubWljcm9zb2Z0LmNv
# bS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MA0GCSqGSIb3
# DQEBCwUAA4ICAQCdVX38Kq3hLB9nATEkW+Geckv8qW/qXBS2Pk5HZHixBpOXPTEz
# tTnXwnE2P9pkbHzQdTltuw8x5MKP+2zRoZQYIu7pZmc6U03dmLq2HnjYNi6cqYJW
# AAOwBb6J6Gngugnue99qb74py27YP0h1AdkY3m2CDPVtI1TkeFN1JFe53Z/zjj3G
# 82jfZfakVqr3lbYoVSfQJL1AoL8ZthISEV09J+BAljis9/kpicO8F7BUhUKz/Aye
# ixmJ5/ALaoHCgRlCGVJ1ijbCHcNhcy4sa3tuPywJeBTpkbKpW99Jo3QMvOyRgNI9
# 5ko+ZjtPu4b6MhrZlvSP9pEB9s7GdP32THJvEKt1MMU0sHrYUP4KWN1APMdUbZ1j
# dEgssU5HLcEUBHG/ZPkkvnNtyo4JvbMBV0lUZNlz138eW0QBjloZkWsNn6Qo3GcZ
# KCS6OEuabvshVGtqRRFHqfG3rsjoiV5PndLQTHa1V1QJsWkBRH58oWFsc/4Ku+xB
# Zj1p/cvBQUl+fpO+y/g75LcVv7TOPqUxUYS8vwLBgqJ7Fx0ViY1w/ue10CgaiQuP
# Ntq6TPmb/wrpNPgkNWcr4A245oyZ1uEi6vAnQj0llOZ0dFtq0Z4+7X6gMTN9vMvp
# e784cETRkPHIqzqKOghif9lwY1NNje6CbaUFEMFxBmoQtB1VM1izoXBm8qGCAtQw
# ggI9AgEBMIIBAKGB2KSB1TCB0jELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
# bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jw
# b3JhdGlvbjEtMCsGA1UECxMkTWljcm9zb2Z0IElyZWxhbmQgT3BlcmF0aW9ucyBM
# aW1pdGVkMSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjpEMDgyLTRCRkQtRUVCQTEl
# MCMGA1UEAxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZaIjCgEBMAcGBSsO
# AwIaAxUAHDn/cz+3yRkIUCJfSbL3djnQEqaggYMwgYCkfjB8MQswCQYDVQQGEwJV
# UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
# ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGlt
# ZS1TdGFtcCBQQ0EgMjAxMDANBgkqhkiG9w0BAQUFAAIFAOnqsKkwIhgPMjAyNDA1
# MTIxMTA2MTdaGA8yMDI0MDUxMzExMDYxN1owdDA6BgorBgEEAYRZCgQBMSwwKjAK
# AgUA6eqwqQIBADAHAgEAAgIPzjAHAgEAAgIRwzAKAgUA6ewCKQIBADA2BgorBgEE
# AYRZCgQCMSgwJjAMBgorBgEEAYRZCgMCoAowCAIBAAIDB6EgoQowCAIBAAIDAYag
# MA0GCSqGSIb3DQEBBQUAA4GBABMlsp2QN0gu+tlC3JCy6YIPb438LlQxC6+nNmBV
# tRjDsLOB8ATpb1dKM5uEB6DTYxMAGA/pIpOSnP7L9EHAxy18XkgURSCUYWiPt2VD
# r3FiMPifbOB7qX86bsyMK8VUMyFeLXy/NOcJePfmC6fxmyur36+jhg9fYPkJ+GSA
# oxJfMYIEDTCCBAkCAQEwgZMwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
# bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jw
# b3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAC
# EzMAAAHcweCMwl9YXo4AAQAAAdwwDQYJYIZIAWUDBAIBBQCgggFKMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAvBgkqhkiG9w0BCQQxIgQg52Ojw85JvjiMIFNk
# 7J/vD4dyGOpbnbu279Hhbn8hFyIwgfoGCyqGSIb3DQEJEAIvMYHqMIHnMIHkMIG9
# BCBTpxeKatlEP4y8qZzjuWL0Ou0IqxELDhX2TLylxIINNzCBmDCBgKR+MHwxCzAJ
# BgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25k
# MR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jv
# c29mdCBUaW1lLVN0YW1wIFBDQSAyMDEwAhMzAAAB3MHgjMJfWF6OAAEAAAHcMCIE
# IC8ey+c2UU1u8F/rB7MdSxl1O3O9ax5gc6ZIV+rcEO92MA0GCSqGSIb3DQEBCwUA
# BIICACm1cdidScTc25fhIYwzLhxP5Lytix28m5gnjYg0pxxCq+hG+DkK6dxD7R0i
# 8OOGeH6npqi+hGPAH3Il8j/tNULZEXPIYPM38QKTNa5zRDbhOLdr/7GIE76Q+5Yh
# DuRJRT7gihScPlYJCuCftN0MxmFodSqvLRuWc0b+fonmq5BdTtg6sXuybM+avhoj
# x0EJwSnsRFraeSva1P7mO4Ov5foWKiZ2z7XKw2ymO98zfiBeUm/v/IuJGXEKJ8BA
# eRA4n6L0RLTNxaE1/yBv+QdJ67er6v4fdGuje7kkBoaDXAKGCubGG9ht16/UjZ34
# BQbUDKee7TSVcwhFh7rFc7jZECdmosGZYfI3QkAw94/ddDBPsL5UtqRQLLqhyG9w
# V9jymf9/QdGwcKugtomngp90RXz2q5q/kTTqs8OR/MG7RE5vG2EPU6jTrsW4PTC7
# ciHNFN8m3VtwcddiOEQf8taErM8AEGNUNfvMeoeIBl8cihfeyNtvEIVa7RUp72au
# MF7T3ZtNc9qKBOUkAnJSFOprMQT7dNK+cKJ0SITsm7mBuMFeiSzTgMAc804BDz7c
# Z366e5wIE/VlcpedF0bbrD0x5UsxI21ki7rDJgReVLrsAD/JCiquFtQnzdioJnBj
# QMKJcGMHSpJMI36JIdo+W8djEaG4drtSPU9fThFWK4F8VamW
# SIG # End signature block
